package org.jp.illg.nora.vr.protocol.model;

import java.nio.ByteBuffer;

import org.jp.illg.dstar.DStarDefines;
import org.jp.illg.dstar.util.DStarUtils;

import lombok.Getter;
import lombok.NonNull;
import lombok.Setter;

public class LoginUser extends NoraVRPacketBase {

	@Getter
	@Setter
	private String loginUserName;

	public LoginUser() {
		super(NoraVRCommandType.LOGINUSR);

		setLoginUserName(DStarDefines.EmptyLongCallsign);
	}

	@Override
	public LoginUser clone() {
		LoginUser copy = null;

		copy = (LoginUser)super.clone();

		copy.loginUserName = this.loginUserName;

		return copy;
	}

	@Override
	protected boolean assembleField(@NonNull ByteBuffer buffer) {
		if(buffer.remaining() < getAssembleFieldLength())
			return false;

		final String userName =
			DStarUtils.formatFullLengthCallsign(getLoginUserName());

		for(int i = 0; i < userName.length() && i < DStarDefines.CallsignFullLength; i++) {
			buffer.put((byte)userName.charAt(i));
		}

		return true;
	}

	@Override
	protected int getAssembleFieldLength() {
		return DStarDefines.CallsignFullLength;
	}

	@Override
	protected boolean parseField(@NonNull ByteBuffer buffer) {
		if(buffer.remaining() < DStarDefines.CallsignFullLength)
			return false;

		final StringBuffer userNameBuffer =
			new StringBuffer(DStarDefines.CallsignFullLength);

		for(int i = 0; i < DStarDefines.CallsignFullLength; i++) {
			userNameBuffer.append((char)buffer.get());
		}

		setLoginUserName(
			DStarUtils.formatFullLengthCallsign(userNameBuffer.toString())
		);

		return true;
	}

	@Override
	public String toString() {
		return toString(0);
	}

	@Override
	public String toString(int indentLevel) {
		if(indentLevel < 0) {indentLevel = 0;}

		StringBuilder sb = new StringBuilder();

		sb.append(super.toString(indentLevel));

		indentLevel += 4;

		String indent = "";
		for(int i = 0; i < indentLevel; i++) {indent += " ";}

		sb.append("\n");

		sb.append(indent);
		sb.append("LoginUserName:");
		sb.append(getLoginUserName());

		return sb.toString();
	}
}
